#!/usr/local/bin/python2.4

#####################################################
# 
# Ian York
# Jan/Feb 2007 
# 
######################################################


import  wx 
import  wx.lib.colourselect as  csel
import  wx.lib.flatnotebook as fnb
import  wx.lib.mixins.listctrl  as  listmix
from    wx.lib.wordwrap import wordwrap 
import  wx.grid as gridlib
import colorsys

from math import pi, cos, sin, radians, degrees , sqrt, atan2
import os
import sys
import time
import copy
import re 
import unicodedata
import random

from elementtree import ElementTree as ET  
from elementtree.SimpleXMLWriter import XMLWriter   
#xml.parsers.expat to handle ET's ExpatError if can't read a non-XML file
import xml.parsers.expat as expat
import Bio
from Bio import GenBank

from Dialogs import *
from GCPanel import *
from ListView import *   

LIGHT_BLUE          = wx.Colour(237,243,254,255)   #(185,211,238,64) 
LIGHT_RED           = wx.Colour(255, 0, 0, 64)
LIGHT_GREEN         = wx.Colour(0, 255, 0, 64)
LIGHT_GREY          = wx.Colour(236,236,236,255)
LINE_GREY           = wx.Colour(190,190,190,255)
DARK_GREY           = wx.Colour(128,128,128,255)
DARKER_GREY         = wx.Colour(64,64,64,255)
TRANSPARENT_RED     = wx.Colour(255,0,0,32)

appPath = sys.argv[0]
pathToResources = os.path.split(appPath)[0] 
        
#----------------------------------------------------------------------- 
#Define TabView(wx.Notebook)  NOTEBOOK STYLE

class TabView(wx.Notebook):
    def __init__(self,parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize,style=wx.NB_BOTTOM):
        wx.Notebook.__init__(self,parent,id,pos,size,style) 
        self.frame = parent
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.onPageChanged)
        
    def onPageChanged(self,evt): 
        mapPage = self.GetCurrentPage()  
        self.frame.toggleLinearMenu()
        
#----------------------------------------------------------------------- 
#Define TabView(wx.FlatNotebook)  FLATNOTEBOOK STYLE

class FlatTabView(fnb.FlatNotebook):
    def __init__(self,parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize,style=fnb.FNB_BOTTOM|fnb.FNB_NO_NAV_BUTTONS|fnb.FNB_VC71|fnb.FNB_X_ON_TAB):
        fnb.FlatNotebook.__init__(self,parent,id,pos,size,style) 
        self.SetNonActiveTabTextColour(DARKER_GREY)
        self.frame = parent
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.onPageChanged)
        
    def onPageChanged(self,evt):
        tabViewIndex = evt.GetSelection() 
        mapPage = self.GetPage(tabViewIndex)  
        self.frame.toggleLinearMenu() 
        
#----------------------------------------------------------------------
#Define SoloFrame(wx.Frame) for stand-alone maps

class SoloFrame(wx.Frame):
    def __init__(self, parent=None, id=wx.ID_ANY, title="SOLO",size=None, pos=None) :
        wx.Frame.__init__(self, parent,id,title,pos,size) 
        
    def GetClientSizeTuple(self):  
        #Return size to match to Notebook size method
        size = self.GetSizeTuple()
        return size
        
#----------------------------------------------------------------------
#Define BaseFrame(wx.Frame) 
        
class BaseFrame(wx.Frame):
    def __init__(self,parent=None, id=wx.ID_ANY, title="XPlasMap",pos=(100,-1),size=(800,850)) :
        wx.Frame.__init__(self, parent,id,title,pos,size) 
        self.SetName("BaseFrame")
        self.size=size 
        self.fragment = False 
        self.unsavedChangesDict = {} 
        self.printData = False
        self.uidList = []
        
        display = wx.Display(0) 
        screenArea = display.GetClientArea()
        #bounding area = (topLeftX,topleftY,bottomRightX, bottomRightY) 
        if screenArea[2] - screenArea[0] > 800:
            self.windowWidth = 800
        else:
            self.windowWidth = screenArea[2] - screenArea[0]
        if screenArea[3] - screenArea[1] > 850:
            self.windowHeight = 850
        else:
            self.windowHeight = screenArea[3] - screenArea[1]
        self.SetSize( (self.windowWidth,self.windowHeight) ) 
        
        self.readPreferences()    
        self.tabView = FlatTabView(self)
        
        self.statusBar = wx.StatusBar(parent=self,id=wx.ID_ANY)
        self.statusBar.SetFieldsCount(1) 
        self.SetStatusBar(self.statusBar)
        
        #Menu IDs
        self.openMenuID             = wx.NewId()
        self.importMenuID           = wx.NewId()
        self.importFromGBMenuID     = wx.NewId()
        self.importFromFastAMenuID  = wx.NewId()
        self.openRecentMenuID       = wx.NewId()
        self.clearRecentMenuID      = wx.NewId()
        self.closeMenuID            = wx.NewId()
        self.saveMenuID             = wx.NewId()
        self.saveAsMenuID           = wx.NewId()  
        self.exportMenuID           = wx.NewId()
        self.importMenuID           = wx.NewId()
        self.saveAsJPGID            = wx.NewId() 
        self.saveAsPNGID            = wx.NewId() 
        self.pageSetupMenuID        = wx.NewId()
        self.printMenuID            = wx.NewId()
        
        self.undoMenuID             = wx.NewId()
        self.newGeneMenuID          = wx.NewId()
        self.newMCSMenuID           = wx.NewId()
        self.newEnzymeMenuID        = wx.NewId()
        self.insertFragmentMenuID   = wx.NewId() 
        self.cutFragmentMenuID      = wx.NewId()
        self.copyFragmentMenuID     = wx.NewId()
        self.editPlasmidMenuID      = wx.NewId()
        self.resetMenuID            = wx.NewId() 
        
        self.newPlasmidMenuID       = wx.NewId()
        self.newLinearMenuID        = wx.NewId()
        self.reverseLinearMenuID    = wx.NewId()
        self.circularizeMenuID      = wx.NewId() 
        self.viewAllAsListMenuID    = wx.NewId()  
        
        #set up menus 
        self.menubar=wx.MenuBar() 
        menu1 = wx.Menu()
        #Auto-insertion into the System menu
        menu1.Append(wx.ID_ABOUT,                       "&About XPlasMap")
        menu1.Append(wx.ID_EXIT,                        "&Quit\tCtrl+Q") 
        menu1.Append(wx.ID_PREFERENCES,                 "&Preferences\tCtrl+,")
          
        menu1.Append(self.openMenuID,                   "&Open\tCtrl+O")
        self.importMenu = wx.Menu()
        self.importMenu.Append(self.importFromGBMenuID, "&from Genbank (.gb files)") 
        self.importMenu.Append(self.importFromFastAMenuID, "&from FastA or text") 
        menu1.AppendMenu(self.importMenuID, "&Import", self.importMenu)
        #set up Open Recent menu
        self.recentFilesIdByFile = {} 
        self.recentFilesFileById = {}
        self.recentFilesMenu = wx.Menu() 
        if len(self.preferences['RecentFiles']) > 0:
            for fileName in self.preferences['RecentFiles']: 
                id = wx.NewId()
                self.recentFilesIdByFile[fileName] = id
                self.recentFilesFileById[id] = fileName
                displayName = os.path.split(fileName)[1]
                self.recentFilesMenu.Insert(0, self.recentFilesIdByFile[fileName], displayName)  
        self.recentFilesMenu.AppendSeparator()   
        self.recentFilesMenu.Append(self.clearRecentMenuID, "Clear recent files")  
        menu1.AppendMenu(self.openRecentMenuID,     "&Open Recent", self.recentFilesMenu)
        menu1.AppendSeparator()
        
        menu1.Append(self.closeMenuID,              "&Close page\tCtrl+W")
        menu1.AppendSeparator()
        menu1.Append(self.saveMenuID,               "&Save\tCtrl+S") 
        menu1.Append(self.saveAsMenuID,             "&Save as\tCtrl+Shift+S") 
        
        exportMenu = wx.Menu() 
        exportMenu.Append(self.saveAsPNGID,         "&to PNG")
        exportMenu.Append(self.saveAsJPGID,         "&to JPG")
        menu1.AppendMenu(self.exportMenuID,         "&Export", exportMenu)
        
        menu1.AppendSeparator()
        menu1.Append(self.pageSetupMenuID,          "&Page setup")
        menu1.Append(self.printMenuID,              "&Print\tCtrl+P") 
        self.menubar.Append(menu1, "&File")
        
        menu2 = wx.Menu()
        menu2.Append(self.undoMenuID,               "&Undo\tCtrl+Z" )
        menu2.AppendSeparator()
        menu2.Append(self.newGeneMenuID,            "&New gene\tCtrl+G")
        menu2.Append(self.newMCSMenuID,             "&New multiple cloning site\tCtrl+Shift+M")
        menu2.Append(self.newEnzymeMenuID,          "&New restriction site\tCtrl+R") 
        menu2.AppendSeparator() 
        menu2.Append(self.cutFragmentMenuID,        "&Cut fragment\tCtrl+X")  
        menu2.Append(self.copyFragmentMenuID,       "&Copy fragment\tCtrl+C") 
        menu2.Append(self.insertFragmentMenuID,     "&Insert fragment\tCtrl+I") 
        menu2.AppendSeparator() 
        menu2.Append(self.resetMenuID,              "&Reset all positions")   
        menu2.AppendSeparator()
        menu2.Append(self.editPlasmidMenuID,        "&Edit plasmid info")
        self.menubar.Append(menu2,"&Edit")  
        
        self.menu3 = wx.Menu()
        self.menu3.Append(self.newPlasmidMenuID,     "&New circular plasmid\tCtrl+N")
        self.menu3.Append(self.newLinearMenuID,      "&New linear DNA\tCtrl+Shift+N") 
        self.menu3.Append(self.reverseLinearMenuID,  "&Draw antiparallel")
        self.menu3.Append(self.circularizeMenuID,    "&Linear <> Circular")
        self.menu3.AppendSeparator()  
        self.menu3.Append(self.viewAllAsListMenuID,  "&List view\tCtrl+L")  
        self.menubar.Append(self.menu3,"&Maps")  
        
        self.SetMenuBar(self.menubar)   
        
        if len(sys.argv) < 2:
            self.menubar.Enable(self.saveAsJPGID,False)
            self.menubar.Enable(self.saveAsPNGID,False)
            self.menubar.Enable(self.printMenuID,False) 
            
            self.menubar.Enable(self.undoMenuID,False)
            self.menubar.Enable(self.newGeneMenuID,False)
            self.menubar.Enable(self.newMCSMenuID,False)
            self.menubar.Enable(self.newEnzymeMenuID,False)
            self.menubar.Enable(self.resetMenuID,False)
            self.menubar.Enable(self.editPlasmidMenuID,False)
            self.menubar.Enable(self.insertFragmentMenuID,False)
            self.menubar.Enable(self.cutFragmentMenuID,False) 
            self.menubar.Enable(self.copyFragmentMenuID,False)
            self.menubar.Enable(self.viewAllAsListMenuID,False) 
            self.menubar.Enable(self.reverseLinearMenuID,False)
            self.menubar.Enable(self.circularizeMenuID,False)  
          
        #File menu events
        self.Bind(wx.EVT_MENU,  self.onAbout,           id=wx.ID_ABOUT)
        self.Bind(wx.EVT_MENU,  self.onQuit,            id=wx.ID_EXIT)
        self.Bind(wx.EVT_MENU,  self.onPreferences,     id=wx.ID_PREFERENCES)
        
        self.Bind(wx.EVT_MENU,  self.onOpenFile,        id=self.openMenuID)
        self.Bind(wx.EVT_MENU,  self.importFromGB,      id=self.importFromGBMenuID)
        self.Bind(wx.EVT_MENU,  self.importFromFastA,   id=self.importFromFastAMenuID)
        for file in self.preferences['RecentFiles']: 
            self.Bind(wx.EVT_MENU,  self.onRecentFile,  id=self.recentFilesIdByFile[file])
        self.Bind(wx.EVT_MENU,  self.clearRecentFiles,  id=self.clearRecentMenuID)
        self.Bind(wx.EVT_MENU,  self.onClosePage,       id=self.closeMenuID)  
        self.Bind(wx.EVT_MENU,  self.onSaveFile,        id=self.saveMenuID)
        self.Bind(wx.EVT_MENU,  self.onSaveAs,          id=self.saveAsMenuID) 
        self.Bind(wx.EVT_MENU,  self.onSaveAsJPG,       id=self.saveAsJPGID)
        self.Bind(wx.EVT_MENU,  self.onSaveAsPNG,       id=self.saveAsPNGID) 
        self.Bind(wx.EVT_MENU,  self.onPageSetup,       id=self.pageSetupMenuID)
        self.Bind(wx.EVT_MENU,  self.onPrint,           id=self.printMenuID)
         
        #Edit menu events
        self.Bind(wx.EVT_MENU,  self.onUndo,            id=self.undoMenuID)
        self.Bind(wx.EVT_MENU,  self.onAddGene,         id=self.newGeneMenuID)
        self.Bind(wx.EVT_MENU,  self.onAddMCS,          id=self.newMCSMenuID)
        self.Bind(wx.EVT_MENU,  self.onAddEnzyme,       id=self.newEnzymeMenuID)   
        self.Bind(wx.EVT_MENU,  self.onDeleteMenu,      id=self.cutFragmentMenuID)  
        self.Bind(wx.EVT_MENU,  self.copyFragmentMenu,  id=self.copyFragmentMenuID) 
        self.Bind(wx.EVT_MENU,  self.onInsertMenu,      id=self.insertFragmentMenuID)  
        self.Bind(wx.EVT_MENU,  self.onReset,           id=self.resetMenuID)  
        self.Bind(wx.EVT_MENU,  self.onEditPlasmid,     id=self.editPlasmidMenuID) 
        
        #Plasmid menu events
        self.Bind(wx.EVT_MENU,  self.getNewPlasmidInfo, id=self.newPlasmidMenuID)
        self.Bind(wx.EVT_MENU,  self.getNewPlasmidInfo, id=self.newLinearMenuID)
        self.Bind(wx.EVT_MENU,  self.onReverseLinear,   id=self.reverseLinearMenuID)
        self.Bind(wx.EVT_MENU,  self.onCircularize,     id=self.circularizeMenuID) 
        self.Bind(wx.EVT_MENU,  self.makeNewListctrl,   id=self.viewAllAsListMenuID) 
        
        #other bindings
        self.Bind(wx.EVT_CLOSE, self.onClosePage)
        self.Bind(wx.EVT_ACTIVATE, self.onActivate)
        
        if len(sys.argv) > 1:
            for fileName in sys.argv[1:]:
                self.openXMLFile(fileName)
  
#-----------------------------------------------------------------------
    def onActivate(self, evt):
        self.setActivated(self)
        evt.Skip()
        
    def setActivated(self, activeWindow):
        self.activeWindow = activeWindow

    def readPreferences(self): 
        #read preferences 
        standardPaths = wx.StandardPaths.Get()
        pathToPreferences = standardPaths.GetUserConfigDir()
        self.preferenceFile = os.path.join(pathToPreferences, "XPlasMap Preferences" )
        self.preferences = {} 
        
        try:
            tree = ET.parse(self.preferenceFile) 
            root = tree.getroot() 
            for elem in list(root):  
                for child in elem.getchildren(): 
                    self.preferences[child.tag] = child.text 
        except: 
            #if can't read preference file, then set everything to default 
            self.preferences['FontFaceName'] = 'Arial'
            self.preferences['FontPointSize'] = 12
            self.preferences['FontWeight'] = wx.FONTWEIGHT_NORMAL
            self.preferences['FontStyle'] = wx.FONTSTYLE_NORMAL
            self.preferences['NotebookStyle'] = 'FlatNoteBook'
            self.preferences['RecentFiles'] = ""
        
        try:
            if self.preferences['RecentFiles'] == "":
                self.preferences['RecentFiles'] = []
            else:
                self.preferences['RecentFiles'] = self.preferences['RecentFiles'].split(",")
        except:
            self.preferences['RecentFiles'] = []
            
    def writePreferences(self):   
        w = XMLWriter(self.preferenceFile)  
        root=w.start("Preferences") 
        fontXML=w.start("Font")
        for (key,value) in self.preferences.items():
            if key[:4] == "Font":
                w.element(key,str( value ) ) 
        w.close(fontXML)  
        generalXML=w.start("General")
        w.element( 'NotebookStyle',self.preferences['NotebookStyle'] ) 
        w.element('RecentFiles', ",".join( self.preferences['RecentFiles'] ) )
        w.close(generalXML) 
        w.close(root) 
        
    def enableMenuItems(self, mapPage): 
        plasmid = mapPage.getPlasmid()
        self.menubar.Enable(self.saveAsJPGID,           True)
        self.menubar.Enable(self.saveAsPNGID,           True)
        self.menubar.Enable(self.printMenuID,           True) 
        self.menubar.Enable(self.newGeneMenuID,         True)
        self.menubar.Enable(self.newMCSMenuID,          True)
        self.menubar.Enable(self.newEnzymeMenuID,       True)
        self.menubar.Enable(self.resetMenuID,           True)
        self.menubar.Enable(self.editPlasmidMenuID,     True)
        self.menubar.Enable(self.insertFragmentMenuID,  True)
        self.menubar.Enable(self.cutFragmentMenuID,     True) 
        self.menubar.Enable(self.copyFragmentMenuID,    True)
        self.menubar.Enable(self.viewAllAsListMenuID,   True) 
        self.menubar.Enable(self.circularizeMenuID,     True)  
        if plasmid['Kind'] == "Linear":
            self.menubar.Enable(self.reverseLinearMenuID,   True) 
        else:
            self.menubar.Enable(self.reverseLinearMenuID,   False) 
 
    #Menu handlers    
    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        
    def onPreferences(self,evt):
        preferencesDialog=PreferencesDialog(self)
        preferencesDialog.ShowModal()
        preferencesDialog.Destroy()
        mapPage = self.tabView.GetCurrentPage()  
        if mapPage != None:
            mapPage.Refresh()
            
    #System menu handlers
    def onQuit(self,evt):   
        shouldQuit = True 
        unsavedPageList = []
        for pageNumber in range( self.tabView.GetPageCount() ):
            page = self.tabView.GetPage(pageNumber)
            if self.unsavedChangesDict[page]:
                unsavedPageList.append(page)
        if len(unsavedPageList) >= 1:  
            if len(unsavedPageList) == 1:
                plasmid = unsavedPageList[0].getPlasmid()
                unsavedError = UnsavedErrorMSG(parent=self, resources=pathToResources,name=plasmid['Name'])
            else:
                unsavedError = UnsavedErrorMSG(parent=self, resources=pathToResources)
            response = unsavedError.ShowModal() 
            if response == wx.ID_YES:
                for pageNumber in range( self.tabView.GetPageCount() ):
                    #SetSelection is deprecated for Notebooks, but ChangeSelection doesn't seem to work for FlatNotebook
                    self.tabView.SetSelection(pageNumber)
                    mapPage = self.tabView.GetCurrentPage() 
                    if mapPage == None:
                        mapPage = self.soloFrame.GetChildren()[0]
                    if self.unsavedChangesDict[mapPage]:
                        self.onSaveFile(evt) 
                shouldQuit = True
            elif response == wx.ID_NO:
                shouldQuit = True
            elif response == wx.ID_CANCEL:
                shouldQuit = False 
        if shouldQuit:
            self.writePreferences()
            if hasattr(self,"listViewFrame"):
                try:
                    self.listViewFrame.Destroy()
                except wx.PyDeadObjectError:
                    pass
                except:
                    raise
            self.Destroy() 
            
    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    #File menu handlers 
        
    def getNewPlasmidInfo(self,evt):     
        mapPage = GCPanel(self.tabView,frame=self) 
        if evt.GetId() == self.newPlasmidMenuID:
            newPlasmidDialog=PlasmidDialog(self, gcPanelInstance=mapPage,DNA="Circular")
        elif evt.GetId() == self.newLinearMenuID:  
            newPlasmidDialog=PlasmidDialog(self, gcPanelInstance=mapPage,DNA="Linear",copied=self.fragment)
        else:
            print "CAN'T UNDERSTAND DNA TYPE"
        newPlasmidDialog.ShowModal()  
        try:
            newPlasmidDialog.Destroy()  
        except wx.PyDeadObjectError:
            pass
        plasmid = mapPage.getPlasmid()  
        if plasmid:
            self.unsavedChangesDict[mapPage] = True 
            if len(plasmid['Name']) > 18:
                tabText = plasmid['Name'][:16] + "..."
            else:
                tabText = plasmid['Name']
            self.tabView.AddPage(mapPage,text=tabText,select=True)  
            self.enableMenuItems(mapPage)
        
    def onReverseLinear(self,evt):
        mapPage = self.tabView.GetCurrentPage() 

        plasmid = mapPage.getPlasmid()
        genes = mapPage.getGenes()
        mcsList = mapPage.getMCSList()
        enzymes = mapPage.getEnzymes() 
        
        reverseGenes = []
        reverseMCSList = []
        reverseEnzymes = []
        for gene in genes:
            reverseGenes.append(gene.copy() )
        for mcs in mcsList:
            reverseMCSList.append(mcs.copy() )
        for enzyme in enzymes:
            reverseEnzymes.append(enzyme.copy() )
        
        newMapPage = GCPanel(self.tabView,frame=self)   
        reversePlasmid = plasmid.copy()
        reversePlasmid['Name'] = plasmid['Name'] + " (Reversed)"
        newMapPage.setPlasmid(reversePlasmid)
        if len(reversePlasmid['Name']) > 18:
            tabText = reversePlasmid['Name'][:16] + "..."
        else:
            tabText = reversePlasmid['Name']
        self.tabView.AddPage(newMapPage,text = tabText,select=True) 
        newMapPage.reverseLinear(reverseGenes, reverseMCSList, reverseEnzymes) 
        
    def onCircularize(self,evt):
        mapPage = self.tabView.GetCurrentPage()  
        mapPage.circularize()
        self.toggleUndoMenu()   
        self.toggleLinearMenu()
            
    def toggleLinearMenu(self):  
        mapPage = self.tabView.GetCurrentPage() 
        plasmid = mapPage.getPlasmid()
        if plasmid['Kind'] == "Linear":
            self.menubar.Enable(self.reverseLinearMenuID,   True)  
            #self.menu3.SetLabel(self.circularizeMenuID, "&Circularize")  
        else:
            self.menubar.Enable(self.reverseLinearMenuID,   False)  
            #self.menu3.SetLabel(self.circularizeMenuID, "&Linearize")   
        
    def onRecentFile(self,evt):
        fileName = self.recentFilesFileById[ evt.GetId() ]     
        self.openXMLFile(fileName) 
        
    def clearRecentFiles(self, evt): 
        for deleteFile in self.preferences['RecentFiles']: 
            deleteId = self.recentFilesIdByFile[deleteFile]
            self.recentFilesMenu.Delete(deleteId)
            self.preferences['RecentFiles'] = []
        
    def onOpenFile(self,evt): 
        """On 'Open' menu item 
        Get the path to the file to open
        Enable menus 
        pass fileName to openXMLFile"""
        openDialog = wx.FileDialog(self, message="Open a plasmid file:", defaultDir="", defaultFile="",  style=wx.OPEN | wx.FD_CHANGE_DIR)
         
        if openDialog.ShowModal()==wx.ID_OK:
            fileNames=openDialog.GetPaths()  
            openDialog.Destroy()   
            
            for fileName in fileNames:
                self.openXMLFile(fileName) 
        else: 
            openDialog.Destroy() 
            
    def openXMLFile(self,fileName):  
        """Read plasmid info stored as XML file (.Xpmp extension)
        fileName is passed from onOpenFile
        Load the XML file info into plasmid, gene, enzyme, and MCS dictionaries"""
        try:    
            mapPage = GCPanel(self.tabView,frame=self)  
            
            tree = ET.parse(fileName) 
            root = tree.getroot() 
            plasmid = {}
            for elem in list(root):     
                if elem.tag=="plasmid": 
                    for child in elem.getchildren():  
                        if child.tag=="Size": 
                            plasmid[child.tag] = float (child.text)
                        else:
                            if child.text=="None": 
                                plasmid[child.tag] = None
                            else:    
                                plasmid[child.tag] = child.text
                    if "ShowComment" not in plasmid.keys():
                        plasmid['ShowComment'] = True
                    if "Kind" not in plasmid.keys():
                        plasmid['Kind'] = "Plasmid"
                    mapPage.setPlasmid(plasmid) 
                    
                elif elem.tag=="gene":
                    gene={}
                    for child in elem.getchildren(): 
                        if child.tag == "Color" or child.tag=="TextColor":
                            #"(255,255,255)" or "(255,255,255,255)"
                            rgb=child.text.replace("(","").replace(")","").replace(","," ")
                            if len(rgb.split())==3:
                                (r,g,b)=rgb.split()  
                                gene[child.tag] = wx.Colour(int(r),int(g),int(b))
                            elif len(rgb.split())==4:
                                (r,g,b,a)=rgb.split()  #a=alpha, transparency
                                gene[child.tag] = wx.Colour(int(r),int(g),int(b),int(a) )  
                        elif child.tag=="Start" or child.tag=="End" or child.tag=="Thickness" or child.tag=="UID" or child.tag=="TextUID":  
                            if child.text=="None":
                                gene[child.tag]=None
                            else:
                                gene[child.tag]=float( child.text )
                        elif child.tag == "Parts":  
                            findDigitsP = re.compile("[-\d]+")
                            digits = re.findall(findDigitsP, child.text)
                            gene['Parts'] = []
                            for i in range(1, len(digits), 2):
                                gene['Parts'].append( (int(digits[i-1]), int(digits[i]) ) ) 
                        else:
                            if child.text=="None":
                                gene[child.tag]=None
                            else:
                                gene[child.tag]= child.text
                    #Update older files' attributes if necessary
                    if 'Parts' not in gene.keys():
                        gene['Parts'] = [ (0, abs(gene['End'] - gene['Start']) ) ] 
                    if 'Style' not in gene.keys():
                        gene['Style'] = "Simple"
                    if "Kind" not in gene.keys():
                        gene["Kind"] = "Gene"  
                    if "TextRadius" not in gene.keys():
                        gene['TextRadius'] = False
                    elif gene['TextRadius'] == "False":
                        gene['TextRadius'] = False
                    if "ShowLocation" not in gene.keys():
                        gene['ShowLocation'] = True
                    elif gene['ShowLocation'] == "True":
                        gene['ShowLocation'] = True
                    elif gene['ShowLocation'] == "False":
                        gene['ShowLocation'] = False
                    if 'Note' not in gene.keys():
                        gene['Note'] = False
                    elif gene['Note'] == "False":
                        gene['Note'] = False
                    elif gene['Note'] == 'True':
                        gene['Note'] = True
                    if 'ShowText' not in gene.keys():
                        gene['ShowText'] = True
                    elif gene['ShowText'] == "True":
                        gene['ShowText'] = True
                    elif gene['ShowText'] == "False":
                        gene['ShowText'] = False
                    #make new UIDs each time
                    gene['TextUID'] = self.getUID() 
                    gene['UID'] = self.getUID()
                    #Convert genes saved as length, to end position 
                    #fixing a legacy condition
                    if 'LengthOrEnd' in gene.keys():
                        if gene['LengthOrEnd'] == "Length":  
                            gene['LengthOrEnd'] = "End"
                            if gene['Direction'] == "Clockwise" or gene['Direction'] == "No Arrow":
                                gene['End'] = gene['Start'] + gene['End']
                            elif gene['Direction'] == "Counterclockwise":
                                gene['End'] = gene['Start'] - gene['End']  
                    else:
                        gene['LengthOrEnd'] = "End"
                    mapPage.setGenes(gene) 
                    
                elif elem.tag=="enzyme":
                    enzyme={}
                    for child in elem.getchildren():
                        enzyme[child.tag] = child.text 
                        if child.tag == "TextXY": #convert string to a tuple of floats
                            t = child.text.replace("(","").replace(")","").replace(",","") 
                            enzyme["TextXY"] = []
                            enzyme["TextXY"].append (float (t.split()[0] ) )
                            enzyme["TextXY"].append (float (t.split()[1] ) )
                            enzyme["TextXY"] = tuple (enzyme["TextXY"] ) 
                        elif child.tag == "Location": 
                            enzyme[child.tag] = float( child.text )
                        elif child.tag == "UID": 
                            enzyme[child.tag] = float( child.text )
                    #Update older files' attributes if necessary
                    if "Kind" not in enzyme.keys():
                        enzyme["Kind"]="Enzyme"
                    if "TextXY" not in enzyme.keys():
                        enzyme["TextXY"]=False
                    if "Show" not in enzyme.keys():
                        enzyme['Show']=True 
                    if "ShowLocation" not in enzyme.keys():
                        enzyme['ShowLocation']=True
                    elif enzyme['ShowLocation']=="True":
                        enzyme['ShowLocation']=True
                    elif enzyme['ShowLocation']=="False":
                        enzyme['ShowLocation']=False                
                    mapPage.setEnzymes(enzyme)        
                    
                elif elem.tag=="mcs":
                    mcs={}
                    for child in elem.getchildren():
                        try:
                            mcs[child.tag]=float( child.text )
                        except ValueError:
                            if child.tag=="mcsCenterXY": #convert string to a tuple of floats
                                t=child.text.replace("(","").replace(")","").replace(",","") 
                                mcs["mcsCenterXY"]=[]
                                mcs["mcsCenterXY"].append (float (t.split()[0] ) )
                                mcs["mcsCenterXY"].append (float (t.split()[1] ) )
                                mcs["mcsCenterXY"]=tuple (mcs["mcsCenterXY"] )  
                            else:
                                mcs[child.tag]=child.text
                    if "Kind" not in mcs.keys():
                        #older files don't have a "Kind" attribute, so update if necessary
                        mcs["Kind"]="MCS"
                    if "mcsCenterXY" not in mcs.keys():
                        mcs["mcsCenterXY"]=False
                    elif  mcs["mcsCenterXY"]=="False":
                         mcs["mcsCenterXY"]=False
                    if "ShowLocation" not in mcs.keys():
                        mcs['ShowLocation']=True
                    elif mcs['ShowLocation']=="True":
                        mcs['ShowLocation']=True
                    elif mcs['ShowLocation']=="False":
                        mcs['ShowLocation']=False
                    mapPage.setMCSList(mcs)   
            
            #Add the map to the notebook and update the list view 
            mapPage.setFileName(fileName)  
            if len(plasmid['Name']) > 18:
                tabText = plasmid['Name'][:18]
            else:
                tabText = plasmid['Name']
            self.tabView.AddPage(mapPage,text=tabText,select=True)  
            mapPage.Refresh()   
            self.refreshListCtrl()   
            
            #Update "Open recent ... " submenu
            if fileName not in self.preferences['RecentFiles']:
                tempID = wx.NewId()
                self.recentFilesIdByFile[fileName] = tempID
                self.recentFilesFileById[tempID] = fileName
                self.preferences['RecentFiles'].append(fileName)
                displayName = os.path.split(fileName)[1]
                self.recentFilesMenu.Insert(0,tempID, displayName)   
                self.Bind(wx.EVT_MENU,  self.onRecentFile,  id=tempID )
                if len(self.preferences['RecentFiles']) > 10:
                    deleteFile = self.preferences['RecentFiles'].pop(0)
                    deleteId = self.recentFilesIdByFile[deleteFile]
                    self.recentFilesMenu.Delete(deleteId) 
             
            self.enableMenuItems(mapPage)     
            
            self.unsavedChangesDict[mapPage] = False
        except IOError: 
            response = wx.MessageBox( message="""Can't find file "%s" """ % fileName,caption="Unable to read file",style=wx.CANCEL)
        except expat.ExpatError:
            response = wx.MessageBox( message=""" "%s" doesn't seem to be a valid XPlasMap file """ % fileName, caption="Unable to read file",style=wx.CANCEL)
        except: 
            response = wx.MessageBox( message="""There is an unknown problem loading file "%s" """ % fileName, caption="Unable to read file",style=wx.CANCEL)
            raise
            print "openXMLFile" , sys.exc_info()
            
    def importFromGB(self,evt): 
        openDialog = wx.FileDialog(self, message="""Import a DNA sequence file in GenBank (".gb") format:""", defaultDir="", defaultFile="",  style=wx.OPEN | wx.FD_CHANGE_DIR)
        if openDialog.ShowModal() == wx.ID_OK:
            filePaths = openDialog.GetPaths()  
            openDialog.Destroy()  
            for filePath in filePaths:
                self.openGBFile(filePath) 
        else: 
            openDialog.Destroy() 
            
    def importFromFastA(self, evt): 
        openDialog = wx.FileDialog(self, message="""Import a DNA sequence file in FastA or plain text format:""", defaultDir="", defaultFile="",  style=wx.OPEN | wx.FD_CHANGE_DIR)
        if openDialog.ShowModal() == wx.ID_OK:
            filePaths = openDialog.GetPaths()  
            openDialog.Destroy()   
            for filePath in filePaths:
                self.openFastAFile(filePath) 
        else: 
            openDialog.Destroy()
        
    def openGBFile(self, filePath):    
        #Parse a GenBank file using BioPython's GenBank module 
        featureParser = GenBank.FeatureParser() 
        gbFile = file(filePath, 'r')
        gbRecord = featureParser.parse(gbFile)
        gbFile.close() 
        sequence = gbRecord.seq.tostring()
        descr = gbRecord.description
        accn = gbRecord.id
        date = gbRecord.annotations['date']   
        importedDNA = {}
        
        importedDNA['Kind'] = "Import" 
        importedDNA['Name'] = descr
        importedDNA['Size'] = len(sequence)
        importedDNA['MadeBy'] = False
        importedDNA['Date'] = date
        importedDNA['Comment'] = "GenBank accession # %s\n%s" % (accn, descr)
        importedDNA['ShowComment'] = True
        importedDNA['UID'] = self.getUID()  
            
        mapPage = GCPanel(self.tabView,frame=self) 
        newPlasmidDialog = PlasmidDialog(self, gcPanelInstance=mapPage,editPlasmid=importedDNA)
        newPlasmidDialog.ShowModal()  
        try:
            newPlasmidDialog.Destroy()  
        except wx.PyDeadObjectError:
            pass 
        mapPage.Refresh()
        
        plasmid = mapPage.getPlasmid() 
        if plasmid:
            self.unsavedChangesDict[mapPage] = True
            if len(plasmid['Name']) > 18:
                tabText = plasmid['Name'][:16] + "..."
            else:
                tabText = plasmid['Name']
            self.tabView.AddPage(mapPage,text=tabText,select=True)   
            self.enableMenuItems(mapPage)
      
        features = gbRecord.features
        featureList = []
        for feature in features:
            if feature.type != 'source':
                featureDict = {}
                featureDict['Type'] = feature.type
                featureDict['Start'] = feature.location.start.position
                featureDict['End'] = feature.location.end.position
                if feature.strand == -1:
                    featureDict['Direction'] = "Counterclockwise"
                    if featureDict['Start'] < featureDict['End']:
                        (featureDict['Start'], featureDict['End']) = (featureDict['End'], featureDict['Start'])   
                else:
                    featureDict['Direction'] = "Clockwise" 
                    
                joinList = []
                if feature.location_operator == "join" :  
                    for subfeature in feature.sub_features:
                        subStart = abs( subfeature.location.start.position - featureDict['Start'] ) 
                        subEnd = abs( subfeature.location.end.position - featureDict['Start'] )
                        joinList.append( (subStart, subEnd) ) 
                else:
                    if featureDict['Start'] < featureDict['End']:
                        joinList = [ (0, featureDict['End'] - featureDict['Start'] ) ]
                    else:
                        joinList = [ (0, featureDict['Start'] - featureDict['End'] ) ]
                tempExonList = joinList
                if featureDict['Direction'] == "Counterclockwise":
                    tempExonList.reverse()
                exonList = []
                for exon in tempExonList:
                    if exon[0] > exon[1]:
                        exonList.append( (exon[1], exon[0]) )
                    else: 
                        exonList.append(exon)
                featureDict['Parts'] = exonList
                if 'gene' in feature.qualifiers.keys():
                    featureDict['Name'] = "%s (%s)" % (feature.qualifiers['gene'][0], featureDict['Type'])
                elif 'note' in feature.qualifiers.keys():
                    note = feature.qualifiers['note'][0].replace("\n", " ")
                    while "  " in note:
                        note = note.replace("  ", " ")
                    featureDict['Name'] = note
                else:
                    featureDict['Name'] = featureDict['Type']  
                if 'note' in feature.qualifiers.keys():
                    note = feature.qualifiers['note'][0].replace("\n", " ")
                    while "  " in note:
                        note = note.replace("  ", " ")
                    featureDict['Note'] = note
                else: 
                    featureDict['Note'] = ""
                featureDict['UID'] =  self.getUID()  
                featureList.append(featureDict)   
        #Call ImportListView to choose which features to import 
        self.importListView = ImportListView(parent=self,size=(self.windowWidth,self.windowHeight-50), tabView=self.tabView, mapPage=mapPage, features=featureList ) 
        self.importListView.Show() 
        
    def openFastAFile(self, filePath):
        fileName = os.path.split(filePath)[1]
        f = file(filePath, 'r')
        sequenceText = f.read()
        f.close() 
        
        newLineP = re.compile("[\n\r]+?") 
        sequenceList = []   
        if sequenceText[0] == ">" or "\n>" in sequenceText:
            #Looks like a FastA file
            sequences = sequenceText.split(">")
            for fasta in sequences:
                try:
                    name = re.split(newLineP, fasta)[0]
                    sequence = "".join( re.split(newLineP, fasta)[1:] )
                    if len(sequence) > 0:
                        sequenceList.append( (name, sequence) )
                except:
                    print sys.exc_info()
        else:
            #Looks like a plain text file
            sequence = re.sub(newLineP, "", sequenceText) 
            sequenceList.append( ("Unknown", sequence) )
        
        for (name, sequence) in sequenceList:
            sequence = sequence.upper() 
            replaceDNAP = re.compile("[\d\s\n/]")
            sequence = re.sub(replaceDNAP, "", sequence)
            notDNAP = re.compile("[^ACGTUN]")
            notDNA = re.findall(notDNAP, sequence)
            if len(notDNA) > 0:
                errorMsg = SequenceErrorMSG(parent=self, resources=pathToResources, name=fileName, xCount=len(notDNA) )
                response = errorMsg.ShowModal()
                if response == wx.ID_OK: 
                    pass
                elif response == wx.ID_CANCEL:
                    break  
            importedDNA = {}
            importedDNA['Kind'] = "Linear" 
            importedDNA['Name'] = name
            importedDNA['Size'] = len(sequence)
            importedDNA['MadeBy'] = False
            importedDNA['Date'] = ""
            importedDNA['Comment'] = "Imported from file %s" %  fileName
            importedDNA['ShowComment'] = True
            importedDNA['UID'] =  self.getUID()   
                
            mapPage = GCPanel(self.tabView,frame=self)  
            mapPage.setPlasmid(importedDNA)
            mapPage.setFileName(fileName) 
            self.unsavedChangesDict[mapPage] = False
            #Add the map to the notebook and update the list view  
            if len(importedDNA['Name']) > 18:
                tabText = importedDNA['Name'][:16] + "..."
            else:
                tabText = importedDNA['Name']
            self.tabView.AddPage(mapPage,text=tabText,select=True)  
            mapPage.Refresh()   
            self.refreshListCtrl()   
            self.enableMenuItems(mapPage)
                           
    def onClosePage(self,evt):  
        """'Close' menu item"""    
        if self.activeWindow.GetName() == "BaseFrame":
            index = self.tabView.GetSelection() 
            if index < 0:
                self.onQuit(evt)
            else: 
                mapPage = self.tabView.GetCurrentPage() 
                if mapPage == None:
                    mapPage = self.soloFrame.GetChildren()[0] 
                if self.unsavedChangesDict[mapPage]:
                    plasmid = mapPage.getPlasmid()
                    name = plasmid['Name']
                    unsavedError = UnsavedErrorMSG(parent=self, resources=pathToResources,name=name)
                    response = unsavedError.ShowModal() 
                    if response == wx.ID_YES:
                        self.onSaveFile(evt) 
                        self.tabView.DeletePage(index) 
                    elif response == wx.ID_NO:
                        self.tabView.DeletePage(index)
                    elif response == wx.ID_CANCEL:
                        pass
                else:                
                    self.tabView.DeletePage(index)  
        else:
            self.activeWindow.Close()
            
    def setUnsavedChanges(self, changes):
        mapPage = self.tabView.GetCurrentPage() 

        self.unsavedChangesDict[mapPage] = changes 
            
    def onSaveFile(self,evt, mapPage=False): 
        """'Save' file item"""
        if not mapPage:
            mapPage = self.tabView.GetCurrentPage() 

        #if the page already has a file associated with it, save without asking for a file
        if mapPage.getFileName(): 
            self.writeXMLFile(mapPage.getFileName())
        elif len(sys.argv) > 1:
            fileName = sys.argv[1] 
        else:
            self.onSaveAs(evt)
            
    def onSaveAs(self,evt): 
        """Get a file path to save plasmid map info as XML
           Then pass that path to saveAs XML """
        mapPage = self.tabView.GetCurrentPage() 

        defaultFileName = mapPage.plasmid['Name'] + ".xpmp"
        saveDialog = wx.FileDialog(self, message="Save as:", defaultDir="", defaultFile=defaultFileName, 
        wildcard="XPlasMap XML (*.xpmp) |*.xpmp", style=wx.SAVE | wx.FD_OVERWRITE_PROMPT | wx.FD_CHANGE_DIR) 
        if saveDialog.ShowModal()==wx.ID_OK:
            fileName=saveDialog.GetPath()   
            saveDialog.Destroy()    
            self.writeXMLFile(fileName) 
        else: 
            saveDialog.Destroy()     
            
    def onSaveAsPostscript(self, evt):  
        """'Save as postscript' menu item
            Calls psPlasmidDraw module which takes plasmid info and converts to a postscript file"""        
        #As of XPlasMap v0.9 this module no longer gets called, 
        #because printing to PDF works much better
        #Leave it in for now in case problems appear
        
        mapPage = self.tabView.GetCurrentPage() 

        plasmid = mapPage.getPlasmid() 
        if plasmid['Kind'] != "Linear":
            import psPlasmidDraw as ps
            genes = mapPage.getGenes()
            mcsList = mapPage.getMCSList()
            enzymes = mapPage.getEnzymes()
            psDraw=ps.psPlasmidDraw(plasmid, genes, mcsList, enzymes)
            psString=psDraw.getPostscriptString()
            
            saveDialog = wx.FileDialog(self, message="Save as:", defaultDir="", defaultFile=mapPage.plasmid['Name'] + ".ps", 
            wildcard=".ps", style=wx.SAVE | wx.FD_OVERWRITE_PROMPT | wx.FD_CHANGE_DIR) 
            if saveDialog.ShowModal() == wx.ID_OK:
                fileName = saveDialog.GetPath()  
                saveDialog.Destroy() 
                f = file(fileName, 'w')
                f.write(psString)
                f.close()
            else: 
                saveDialog.Destroy()    
        else:
            errorMsg = wx.MessageBox("Sorry, at this time export to postscript only works for circular plasmids.", "Can't make .ps file")
             
    def onSaveAsPNG(self,evt):
        """'Save as PNG' menu item""" 
        mapPage = self.tabView.GetCurrentPage() 
        (width, height) = mapPage.GetSizeTuple()
        bitmap = wx.EmptyBitmap(width, height )
        memoryDC = wx.MemoryDC(bitmap)  
        mapPage.doDrawing(memoryDC)  
        saveDialog = wx.FileDialog(self, message="Save as:", defaultDir="", defaultFile=mapPage.plasmid['Name'] + ".PNG", 
        wildcard=".PNG", style=wx.SAVE | wx.FD_OVERWRITE_PROMPT | wx.FD_CHANGE_DIR) 
        if saveDialog.ShowModal() == wx.ID_OK:
            fileName = saveDialog.GetPath()  
            saveDialog.Destroy()  
            bitmap.SaveFile(fileName, wx.BITMAP_TYPE_PNG)
        else: 
            saveDialog.Destroy()       
    
    def onSaveAsJPG(self,evt):
        """'Save as JPG menu item"""
        mapPage = self.tabView.GetCurrentPage()   
        (width, height) = mapPage.GetSizeTuple()
        bitmap = wx.EmptyBitmap(width, height )
        memoryDC = wx.MemoryDC(bitmap) 
        memoryDC.SetPen(wx.NullPen ) 
        memoryDC.SetBrush(wx.WHITE_BRUSH)
        memoryDC.DrawRectangle(0, 0, width, height )
        mapPage.doDrawing(memoryDC)   
        saveDialog = wx.FileDialog(self, message="Save as:", defaultDir="", defaultFile=mapPage.plasmid['Name'] + ".JPG", 
        wildcard=".JPG", style=wx.SAVE | wx.FD_OVERWRITE_PROMPT | wx.FD_CHANGE_DIR) 
        if saveDialog.ShowModal() == wx.ID_OK:
            fileName = saveDialog.GetPath()  
            saveDialog.Destroy()  
            bitmap.SaveFile(fileName, wx.BITMAP_TYPE_JPEG)
        else: 
            saveDialog.Destroy()       
        
    def writeXMLFile(self,fileName ):
        """FileName passed from onsaveas
           Use elementtree module to write data as an XML file"""
        w = XMLWriter(fileName)
        mapPage = self.tabView.GetCurrentPage() 

        plasmid=mapPage.getPlasmid()
        genes=mapPage.getGenes()
        mcsList=mapPage.getMCSList()
        enzymes=mapPage.getEnzymes()
        try:
            root=w.start("XPlasMap") 
            plasmidXML=w.start("plasmid")
            for (key,value) in plasmid.items():
                w.element(key,str( value ) ) 
            w.close(plasmidXML)
            for gene in genes:
                geneXML=w.start("gene")
                for (key,value) in gene.items():
                    w.element(key,str( value ) ) 
                w.close(geneXML)
            for mcs in mcsList:
                mcsXML=w.start("mcs")
                for (key,value) in mcs.items():
                    w.element(key,str( value ) ) 
                w.close(mcsXML)
            for enzyme in enzymes:
                enzymeXML=w.start("enzyme")
                for (key,value) in enzyme.items():
                    w.element(key,str( value ) ) 
                w.close(enzymeXML)
            w.close(root)
        except:
            response = wx.MessageBox(message="Unknown problem trying to write file",caption="Couldn't save file",style=wx.OK)
        
        mapPage.setFileName(fileName) 
        self.unsavedChangesDict[mapPage] = False
        
        #Update "Open recent ... " submenu
        if fileName not in self.preferences['RecentFiles']:
            tempID = wx.NewId()
            self.recentFilesIdByFile[fileName] = tempID
            self.recentFilesFileById[tempID] = fileName
            self.preferences['RecentFiles'].append(fileName)
            displayName = os.path.split(fileName)[1]
            self.recentFilesMenu.Insert(0,tempID, displayName)   
            self.Bind(wx.EVT_MENU,  self.onRecentFile,  id=tempID )
            if len(self.preferences['RecentFiles']) > 10:
                deleteFile = self.preferences['RecentFiles'].pop(0)
                deleteId = self.recentFilesIdByFile[deleteFile]
                self.recentFilesMenu.Delete(deleteId)
            
    def onPageSetup(self,evt):
        mapPage = self.tabView.GetCurrentPage() 
        pageSetup = TestPrintPanel(self, mapPage )
        self.printData = pageSetup.OnPageSetup(evt) 
            
    def onPrint(self,evt):  
        mapPage = self.tabView.GetCurrentPage()
        printDialog = TestPrintPanel(self, mapPage, self.printData)
        printDialog.OnDoPrint(evt) 
        printDialog.Destroy()
     
    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    #Edit menu handlers  
    
    def onUndo(self,evt): 
        mapPage = self.tabView.GetCurrentPage() 
 
        #get the most recent plasmid state 
        undo = mapPage.restoreLastChange()
        if undo: 
            (backupPlasmid, backupMCSList, backupGenes, backupEnzymes) = undo
            currentMCSList = copy.copy( mapPage.getMCSList() )
            currentGenes = copy.copy( mapPage.getGenes() )
            currentEnzymes = copy.copy( mapPage.getEnzymes() ) 
            mapPage.setPlasmid(backupPlasmid)  
            for currentGene in currentGenes:
                mapPage.deleteGene(currentGene['UID'], backup=False)
            for currentMCS in currentMCSList:
                mapPage.deleteMCS(currentMCS['UID'], backup=False) 
            for currentEnzyme in currentEnzymes:
                mapPage.deleteEnzyme(currentEnzyme['UID'], backup=False) 
            for gene in backupGenes:
                mapPage.setGenes(gene)
            for mcs in backupMCSList:
                mapPage.setMCSList(mcs)
            for enzyme in backupEnzymes:
                mapPage.setEnzymes(enzyme) 
            self.toggleUndoMenu() 
    
    def toggleUndoMenu(self):  
        mapPage = self.tabView.GetCurrentPage()  
        undoListLength = mapPage.getBackupsForUndo()  
        if undoListLength > 0: 
            self.menubar.Enable(self.undoMenuID,True)  
        else: 
            self.menubar.Enable(self.undoMenuID,False)  
    
    def onAddGene(self,evt):  
        mapPage = self.tabView.GetCurrentPage() 

        geneDialog=GeneDialog(self, gcPanelInstance=mapPage) 
        geneDialog.ShowModal()   
        try:
            geneDialog.Destroy()   
        except wx.PyDeadObjectError:
            pass  
        self.toggleUndoMenu() 
            
    def appendToGenes(self,gene): 
        mapPage = self.tabView.GetCurrentPage() 
 
        mapPage.setGenes(gene)   
        mapPage.drawGene(gene) 
        
    def onAddMCS(self,evt):
        mapPage = self.tabView.GetCurrentPage() 

        mcsDialog=MCSDialog(self,gcPanelInstance=mapPage)
        mcsDialog.ShowModal()    
        try:
            mcsDialog.Destroy()   
        except wx.PyDeadObjectError:
            pass  
        self.toggleUndoMenu() 
        
    def appendToMCS(self,mcs):  
        mapPage = self.tabView.GetCurrentPage() 

        mapPage.setMCSList(mcs)  
        mapPage.Refresh() 
        self.refreshListCtrl() 
        self.toggleUndoMenu() 
        
    def onAddEnzyme(self,evt): 
        mapPage = self.tabView.GetCurrentPage()  
        enzymeDialog=EnzymeDialog(self,gcPanelInstance=mapPage)
        enzymeDialog.ShowModal()    
        try:
            enzymeDialog.Destroy()   
        except wx.PyDeadObjectError:
            pass  
        self.toggleUndoMenu() 
        
    def onReset(self, evt): 
        mapPage = self.tabView.GetCurrentPage()  
        mapPage.resetToDefaultPositions()
        self.toggleUndoMenu()   
    
    def onDeleteMenu(self,evt):
        mapPage = self.tabView.GetCurrentPage()  
        undoListLength = mapPage.getBackupsForUndo()  
        plasmid = mapPage.getPlasmid()
        deleteDialog = DeleteDialog(parent=self, gcPanelInstance=mapPage, plasmid=plasmid)
        response = deleteDialog.ShowModal()   
        if response == wx.ID_DELETE or response == wx.ID_CUT : 
            self.toggleUndoMenu()  
        try:
            deleteDialog.Destroy()   
        except wx.PyDeadObjectError:
            pass   
        
    def onInsertMenu(self,evt): 
        mapPage = self.tabView.GetCurrentPage()  
        insertFragmentDialog = InsertFragmentDialog(parent=self,gcPanelInstance=mapPage,copied=self.fragment )
        response = insertFragmentDialog.ShowModal() 
        self.toggleUndoMenu() 

    def copyFragmentMenu(self,evt): 
        mapPage = self.tabView.GetCurrentPage()  
        copyFragmentDialog = CopyFragmentDialog(parent=self,gcPanelInstance=mapPage )
        response = copyFragmentDialog.ShowModal() 
        self.toggleUndoMenu() 
        
    def onEditPlasmid(self, evt):
        mapPage = self.tabView.GetCurrentPage() 
        plasmid = mapPage.getPlasmid()
        mapPage.backupsForUndo()
        plasmidDialog = PlasmidDialog(parent=self,gcPanelInstance=mapPage,editPlasmid=plasmid)
        response = plasmidDialog.ShowModal() 
        if response == wx.ID_CANCEL:
            self.onUndo(evt)
        self.toggleUndoMenu()   

        
    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    #View menu handlers 
    
    def makeSoloFrame(self, evt):
        (xPos, yPos) = self.GetPositionTuple()
        while self.tabView.GetPageCount() > 0:
            xPos = xPos + 25
            self.soloFrame = SoloFrame(parent=self, size=(self.windowWidth,self.windowHeight), pos=(xPos, yPos) )        
 
            mapPage = self.tabView.GetCurrentPage()
            plasmid = mapPage.getPlasmid()
            genes = mapPage.getGenes()
            mcsList = mapPage.getMCSList()
            enzymes = mapPage.getEnzymes() 
            newMapPage = GCPanel(parent=self.soloFrame,frame=self )  
            newMapPage.setPlasmid(plasmid)
            for gene in genes:
                newMapPage.setGenes(gene.copy() )
            for mcs in mcsList:
                newMapPage.setMCSList(mcs.copy() )
            for enzyme in enzymes:
                newMapPage.setEnzymes(enzyme.copy() ) 
            pageIndex = self.tabView.GetSelection()
            title = self.tabView.GetPageText(pageIndex)
            self.soloFrame.Show()  
            self.tabView.DeletePage(pageIndex)  
            self.tabView.AdvanceSelection()
        self.menubar.Enable(self.tabViewMenuID,True)
        self.tabView.Destroy()
        self.Hide()
        
    def makeTabView(self, evt): 
        print self.GetChildren()
    
    def makeNewListctrl(self,evt):
        #set height 50 shorter to account for the toolbar
        self.listViewFrame=ListViewFrame(parent=self,size=(self.windowWidth,self.windowHeight-50) , tabView=self.tabView,mapPage = self.tabView.GetCurrentPage() ) 
        self.listViewFrame.Show() 
        
    #-------------------------------------------------------------------
   
    def refreshListCtrl(self): 
        if hasattr(self,"listViewFrame"):
            try:
                self.listViewFrame.refreshListCtrl()
            except wx.PyDeadObjectError:
                pass
            except:
                raise
    
    def deleteChecked(self,evt):
        pass
        
    def toggleLocations(self,evt):
        pass
        
    def showHideChecked(self,evt):
        pass 
        
    def getPreferences(self):
        return (self.preferences, self.preferenceFile)
 
    def setSavedFragment(self, fragment, fragmentGenes, fragmentMCSList, fragmentEnzymes):
        self.fragment = fragment
        self.fragmentGenes = fragmentGenes
        self.fragmentMCSList = fragmentMCSList 
        self.fragmentEnzymes = fragmentEnzymes 
 
    def getSavedFragment(self):
        return (self.fragment,   self.fragmentGenes, self.fragmentMCSList, self.fragmentEnzymes)
        
    def setStatusText(self,text):
        self.statusBar.SetStatusText(text)
        
    def getUID(self): 
        while True:
            uid = random.randrange(1000000,999999999)
            if uid not in self.uidList:
                break
        return uid
            

    def onAbout(self,evt):
        info = wx.AboutDialogInfo()
        info.Name = "XPlasMap"
        info.Version = "0.9"
        info.Copyright = "(C) 2007 Ian York"
        info.Description = """
 XPlasMap is free for personal use.
 
 Distribution of XPlasMap is allowed and encouraged, but only 
 as a complete package (including any documentation, read-me,
 and copyright files). Use XPlasMap at your own risk. The author 
 is not responsible for any damage, annoyance, or waste of time 
 or materials resulting from the use of XPlasMap or incorrect 
 information provided by XPlasMap.

 XPlasMap was developed by Ian York (iayork@gmail.com), 
 using Python and wx.Python:
    http://www.python.org
    http://www.wxPython.org 

Additional modules used in XPlasMap:
    ElementTree (http://effbot.org/zone/element-index.htm)
    BioPython (http://biopython.org/)
    BioPython uses the eGenix mx extensions for Python
    (http://www.egenix.com/www2002/python/eGenix-mx-Extensions.html)
 
 Thanks to Paul Tomblin and Clive Feather for geometry advice.
 
 Please send feedback to iayork@iayork.com.\n\n"""
 
        info.WebSite = ("http://www.panix.com/~iayork/XPlasMap", "XPlasMap home page")
        #info.Developers = [ "Ian York (iayork@gmail.com)" ]

        info.License ="""
Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
   
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."""

        wx.AboutBox(info)
    
        
    
#----------------------------------------------------------------------
#Printing frameworks
#Basically lifted from the wxPython demo

class MyPrintout(wx.Printout):
    def __init__(self, mapPage):
        wx.Printout.__init__(self)
        self.mapPage = mapPage 
    
    def GetPageInfo(self):
        #print one page
        return (1, 1, 1, 1) #page minimum, page max, required page range min, required page range max
    
    def OnPrintPage(self, page ):  
        printDC = self.GetDC()
        (maxX, maxY) = self.mapPage.GetSizeTuple() 
        buff = wx.EmptyBitmap(maxX, maxY )
        memoryDC = wx.MemoryDC(buff) 
     
        # Set scaling factors... 
        (maxX, maxY) = self.mapPage.GetSizeTuple()   
        marginX = 50
        marginY = 50 
        maxX = maxX + (2 * marginX)
        maxY = maxY + (2 * marginY) 
        (w, h) = printDC.GetSizeTuple() 
        scaleX = float(w) / maxX
        scaleY = float(h) / maxY  
        actualScale = min(scaleX, scaleY)
    
        # Calculate the position on the DC for centering the graphic
        posX = (w - (maxX * actualScale)) / 2.0
        posY = (h - (maxY * actualScale)) / 2.0
    
        # Set the scale and origin
        printDC.SetUserScale(actualScale, actualScale)
        printDC.SetDeviceOrigin(int(posX), int(posY))
      
        self.mapPage.doDrawing(memoryDC)  
        printDC.Blit(0, 0, maxX, maxY, memoryDC, 0, 0) 
        return True 


#---------------------------------------------------------------------- 

class TestPrintPanel(wx.Panel):
    def __init__(self, parent,  mapPage, printData=False):
        wx.Panel.__init__(self, parent, -1)  
        self.mapPage = mapPage 
        self.parent = parent
        if printData:
            self.printData = printData 
        else:
            self.printData = wx.PrintData()
            self.printData.SetPaperId(wx.PAPER_LETTER)
            self.printData.SetPrintMode(wx.PRINT_MODE_PRINTER)
        
        self.box = wx.BoxSizer(wx.VERTICAL)
        self.box.Add(self.mapPage, 1, wx.GROW)
    
        subbox = wx.BoxSizer(wx.HORIZONTAL)
        btn = wx.Button(self, -1, "Page Setup")
        self.Bind(wx.EVT_BUTTON, self.OnPageSetup, btn)
        subbox.Add(btn, 1, wx.GROW | wx.ALL, 2)
    
        btn = wx.Button(self, -1, "Print Preview")
        self.Bind(wx.EVT_BUTTON, self.OnPrintPreview, btn)
        subbox.Add(btn, 1, wx.GROW | wx.ALL, 2)
    
        btn = wx.Button(self, -1, "Print")
        self.Bind(wx.EVT_BUTTON, self.OnDoPrint, btn)
        subbox.Add(btn, 1, wx.GROW | wx.ALL, 2)
    
        self.box.Add(subbox, 0, wx.GROW)
    
        self.SetAutoLayout(True)
        self.SetSizer(self.box)
    
    def OnPageSetup(self, evt):
        psdd = wx.PageSetupDialogData(self.printData)
        psdd.CalculatePaperSizeFromId()
        dlg = wx.PageSetupDialog(self, psdd)
        dlg.ShowModal() 
        # this makes a copy of the wx.PrintData instead of just saving
        # a reference to the one inside the PrintDialogData that will
        # be destroyed when the dialog is destroyed
        self.printData = wx.PrintData( dlg.GetPageSetupDialogData().GetPrintData() )   
        return self.printData
        dlg.Destroy()
    
    def OnPrintPreview(self, event):
        data = wx.PrintDialogData(self.printData)
        printout = MyPrintout(self.mapPage)
        printout2 = MyPrintout(self.mapPage)
        self.preview = wx.PrintPreview(printout, printout2, data)
    
        pfrm = wx.PreviewFrame(self.preview, self.parent, "Print preview")
    
        pfrm.Initialize()
        pfrm.SetPosition(self.parent.GetPosition())
        pfrm.SetSize(self.parent.GetSize())
        pfrm.Show(True)
    
    def OnDoPrint(self, event): 
        pdd = wx.PrintDialogData(self.printData)
        pdd.SetToPage(1)
        printer = wx.Printer(pdd)
        printout = MyPrintout(self.mapPage ) 
        printResponse = printer.Print(self.parent, printout, True) 
        if not printResponse:
            wx.MessageBox("Sorry, unable to print", "Problem printing", wx.OK)
        else:
            self.printData = wx.PrintData( printer.GetPrintDialogData().GetPrintData() )
        printout.Destroy() 
        
         

class BaseApp(wx.App):
    def __init__(self):
        wx.App.__init__(self,redirect=False) 
        self.frame = BaseFrame( )
        self.frame.Show()  
        
    def MacOpenFile(self, filename):
        self.frame.openXMLFile(filename)
         
app = BaseApp() #wx.PySimpleApp()
app.MainLoop()